-- Nicht relationale Daten - Demo 2
-- FileTable - Verwendung


USE SNEK_FileTable
GO


----------------------------------
-- Dateien und Dokumente ermitteln
----------------------------------

-- Alle Dokumente
SELECT * FROM dbo.Dokumente

-- Dokumente mit Dateierweiterung txt, docx, xlsx
SELECT * FROM dbo.Dokumente WHERE file_type IN ('txt', 'docx', 'xlsx')

-- Dokumente, die heute erstellt wurden
SELECT * FROM dbo.Dokumente WHERE creation_time >= Cast(getdate() As date)

-- Dokumente, die in den letzten 2 Minuten gendert wurden
SELECT * FROM dbo.Dokumente WHERE last_write_time >= DATEADD(MINUTE, -2, getutcdate())

-- Dokumente, auf die in den letzten 2 Minuten zugegriffen wurde
SELECT * FROM dbo.Dokumente WHERE last_access_time >= DATEADD(MINUTE, -2, getutcdate())


--------------------------------
-- Dokumente per T-SQL verwalten
--------------------------------

-- Dokument umbenennen
UPDATE	dbo.Dokumente SET [name] = 'SNEK 2012.docx' WHERE [name] = 'SNEK - Kopie.docx'
SELECT * FROM dbo.Dokumente WHERE file_type = 'docx' and parent_path_locator Is Null

-- Dokument lschen
DELETE FROM dbo.Dokumente WHERE [name] = 'SNEK 2012.docx'
SELECT * FROM dbo.Dokumente WHERE file_type = 'docx' and parent_path_locator Is Null


----------------------------------------
-- Verzeichnisse ermitteln und verwalten
----------------------------------------

-- Alle Verzeichnisse 
SELECT * FROM dbo.Dokumente WHERE is_directory = 1

-- Verzeichnis anlegen
INSERT INTO dbo.Dokumente (Name, Is_Directory) VALUES ('Schulung', 1)
SELECT * FROM dbo.Dokumente WHERE is_directory = 1

-- Verzeichnis umbenennen
UPDATE dbo.Dokumente SET Name = 'Schulungen' WHERE Name = 'Schulung' AND is_directory = 1
SELECT * FROM dbo.Dokumente WHERE is_directory = 1

-- Verzeichnis lschen
DELETE FROM dbo.Dokumente WHERE Name = 'Schulungen' AND Is_Directory = 1
SELECT * FROM dbo.Dokumente WHERE is_directory = 1


-- Inhalt vom Verzeichnis 'Workshops'
SELECT	D2.Name, D2.Is_Directory 
FROM	dbo.Dokumente D1 INNER JOIN dbo.Dokumente D2 ON D1.path_locator = D2.parent_path_locator
WHERE	D1.[name] = 'Workshops'


----------------------------------------
-- Dokumente und Verzeichnisse ermitteln
----------------------------------------

-- Alle Dokumente inklusive Name, relativem Pfad zum Dokument und Pfad der FileTable
SELECT	name, is_directory,
		'File:' + file_stream.GetFileNamespacePath(1, Null) As Pfad,
		FileTableRootPath('dbo.Dokumente') As FileTablePfad 
FROM	dbo.Dokumente

-- Statistiken zum Verzeichnis der FileTable
SELECT	COUNT(CASE WHEN is_directory = 0 THEN [Name] END) AS Dokumente,
		COUNT(CASE WHEN is_directory = 1 THEN [Name]  END) As Verzeichnisse,
		COUNT(DISTINCT file_type) As Dateitypen
FROM	dbo.Dokumente