-- Nicht relationale Daten - Demo 6
-- Volltextsuche in Dateien und Dokumenten - Semantische Suche


--------------------------------------------------------
-- Semantische Suche in SQL Server-Instanz konfigurieren
--------------------------------------------------------

USE [master]
GO

-- "SemanticLanguageDatabase.msi" installieren!
-- Download siehe PDF vom Vortrag

-- Datenbank "semanticsdb" registrieren
EXEC sp_fulltext_semantic_register_language_statistics_db @dbname = N'semanticsdb';
GO

-- Untersttzte Sprachen anzeigen
SELECT * FROM sys.fulltext_semantic_languages;
GO


-----------------------------------------------------
-- Volltextindex zur semantischen Suche konfigurieren
-----------------------------------------------------

USE SNEK_FileTable
GO

-- Alten Index lschen
DROP FULLTEXT INDEX ON dbo.Dokumente

-- Volltextindex mit STATISTICAL_SEMANTICS neu erstellen
-- Besser manuell ber das SSMS anlegen!!
CREATE FULLTEXT INDEX ON dbo.Dokumente
(
    name LANGUAGE 1031,
    file_stream TYPE COLUMN file_type LANGUAGE 1031 STATISTICAL_SEMANTICS
)
KEY INDEX PK__Dokument__5A5B77D50547F93D
ON vtDokumente
WITH
    CHANGE_TRACKING AUTO,
    STOPLIST = SYSTEM
GO


----------------------------------------------
-- Semantische Suche in Dateien und Dokumenten
----------------------------------------------

USE SNEK_FileTable
GO


-- Schlsselwrter zu einem Dokument 
DECLARE @Dokument hierarchyid
SELECT @Dokument = path_locator from dbo.Dokumente where name = 'migration.docx';

SELECT	Tags.keyphrase, Tags.score
FROM	SEMANTICKEYPHRASETABLE(dbo.Dokumente, file_stream, @Dokument) As Tags
ORDER BY Tags.score DESC;
GO


-- Dokumente mit dem Schlsselwort "Migration" 
SELECT	Doks.name, Doks.file_type,
		file_stream.GetFileNamespacePath(1, Null) As Pfad
FROM	dbo.Dokumente Doks
		INNER JOIN SEMANTICKEYPHRASETABLE(dbo.Dokumente, file_stream) As Tags
		ON Doks.path_locator = Tags.document_key
WHERE	Tags.keyphrase = 'Migration'
ORDER BY Tags.Score DESC;
GO


-- hnliche Dokumente 
DECLARE @Dokument hierarchyid
SELECT @Dokument = path_locator from dbo.Dokumente where name = 'ssrs.docx';

SELECT	Doks.name, Doks.file_type, DoksWie.score 
FROM	dbo.Dokumente Doks 
		INNER JOIN SEMANTICSIMILARITYTABLE(dbo.Dokumente, file_stream, @Dokument) As DoksWie
		ON Doks.path_locator = DoksWie.matched_document_key 
ORDER BY DoksWie.Score DESC
GO


-- hnlichkeiten zweier Dokumente 
DECLARE @Dokument hierarchyid
SELECT @Dokument = path_locator from dbo.Dokumente where name = 'ssrs.docx';

DECLARE @DocWie hierarchyid
SELECT @DocWie = path_locator FROM dbo.Dokumente WHERE name = 'migration.docx';

SELECT	Vergleich.keyphrase, Vergleich.score
FROM	SEMANTICSIMILARITYDETAILSTABLE(dbo.Dokumente, file_stream, @Dokument, file_stream, @DocWie) As Vergleich
ORDER BY Vergleich.Score DESC
GO