-- T-SQL - Demo 1
-- Fensterfunktionen


USE SNEK
GO

-----------------------------
-- Einfache Fensterfunktionen
-----------------------------

-- Aggregate inklusive Detailinformationen zu zwei Bestellungen 
SELECT	[Bestell-Nr], [Artikel-Nr], Anzahl, 
		SUM(Anzahl) OVER(PARTITION BY [Bestell-Nr]) AS 'SummeAnzahl',
		AVG(Anzahl) OVER(PARTITION BY [Bestell-Nr]) AS 'DurchschnittAnzahl',
		COUNT(Anzahl) OVER(PARTITION BY [Bestell-Nr]) AS 'GesamtAnzahl',
		MIN(Anzahl) OVER(PARTITION BY [Bestell-Nr]) AS 'MinAnzahl',
		MAX(Anzahl) OVER(PARTITION BY [Bestell-Nr]) AS 'MaxAnzahl'
FROM	dbo.Bestelldetails
WHERE	[Bestell-Nr] BETWEEN 11075 AND 11076


-- Aggregate inklusive Detailinformationen zu den Bestellungen vom Kunden "ALFKI" 
SELECT	[Kunden-Code], B.[Bestell-Nr], Bestelldatum, [Artikel-Nr], Anzahl, 
		SUM(Anzahl) OVER(PARTITION BY B.[Kunden-Code]) AS 'SummeAnzahl',
		AVG(Anzahl) OVER(PARTITION BY B.[Kunden-Code]) AS 'DurchschnittAnzahl',
		COUNT(Anzahl) OVER(PARTITION BY B.[Kunden-Code]) AS 'GesamtAnzahl',
		MIN(Anzahl) OVER(PARTITION BY B.[Kunden-Code]) AS 'MinAnzahl',
		MAX(Anzahl) OVER(PARTITION BY B.[Kunden-Code]) AS 'MaxAnzahl'
FROM	dbo.Bestelldetails BD INNER JOIN dbo.Bestellungen B ON BD.[Bestell-Nr] = B.[Bestell-Nr] 
WHERE	[Kunden-Code] = 'ALFKI'


-- Aggregate inklusive Detailinformationen zu Bestellungen der jeweiligen Kunden 
SELECT	[Kunden-Code], B.[Bestell-Nr], Bestelldatum, [Artikel-Nr], Anzahl, 
		SUM(Anzahl) OVER(PARTITION BY B.[Kunden-Code]) AS 'SummeAnzahl',
		AVG(Anzahl) OVER(PARTITION BY B.[Kunden-Code]) AS 'DurchschnittAnzahl',
		COUNT(Anzahl) OVER(PARTITION BY B.[Kunden-Code]) AS 'GesamtAnzahl',
		MIN(Anzahl) OVER(PARTITION BY B.[Kunden-Code]) AS 'MinAnzahl',
		MAX(Anzahl) OVER(PARTITION BY B.[Kunden-Code]) AS 'MaxAnzahl'
FROM	dbo.Bestelldetails BD INNER JOIN dbo.Bestellungen B ON BD.[Bestell-Nr] = B.[Bestell-Nr] 


-- Summe der Anzahl der jeweiligen Positionen pro Bestellung 
SELECT	[Bestell-Nr], [Artikel-Nr], Anzahl, 
		SUM(Anzahl) OVER(PARTITION BY [Bestell-Nr] ORDER BY [Artikel-Nr]) As SumAnzahl
FROM	dbo.Bestelldetails
WHERE	[Bestell-Nr] BETWEEN 11075 AND 11076


-----------------------------
-- Fensterfunktionen mit ROWS
-----------------------------

-- Summe der Anzahl der aktuellen und der nchsten Position pro Bestellung 
SELECT	[Bestell-Nr], [Artikel-Nr], Anzahl, 
		SUM(Anzahl) OVER(PARTITION BY [Bestell-Nr] 
						 ORDER BY [Artikel-Nr]
						 ROWS BETWEEN CURRENT ROW AND 1 FOLLOWING) As SumAnzahl
FROM	dbo.Bestelldetails
WHERE	[Bestell-Nr] BETWEEN 11075 AND 11076


-- Summe der Anzahl der aktuellen und der vorherigen Position pro Bestellung 
SELECT	[Bestell-Nr], [Artikel-Nr], Anzahl, 
		SUM(Anzahl) OVER(PARTITION BY [Bestell-Nr] 
						 ORDER BY [Artikel-Nr]
						 ROWS BETWEEN 1 PRECEDING AND CURRENT ROW) As SumAnzahl
FROM	dbo.Bestelldetails
WHERE	[Bestell-Nr] BETWEEN 11075 AND 11076


-- Summe der Anzahl der vorherigen und der nchsten Position pro Bestellung 
SELECT	[Bestell-Nr], [Artikel-Nr], Anzahl, 
		SUM(Anzahl) OVER(PARTITION BY [Bestell-Nr] 
						 ORDER BY [Artikel-Nr]
						 ROWS BETWEEN 1 PRECEDING AND 1 FOLLOWING) As SumAnzahl
FROM	dbo.Bestelldetails
WHERE	[Bestell-Nr] BETWEEN 11075 AND 11076


-- Summe der Anzahl der nchsten und bernchsten Position pro Bestellung 
SELECT	[Bestell-Nr], [Artikel-Nr], Anzahl, 
		SUM(Anzahl) OVER(PARTITION BY [Bestell-Nr] 
						 ORDER BY [Artikel-Nr]
						 ROWS BETWEEN 1 FOLLOWING AND 2 FOLLOWING) As SumAnzahl
FROM	dbo.Bestelldetails
WHERE	[Bestell-Nr] BETWEEN 11075 AND 11076


-- Summe der Anzahl der aktuellen und nchsten Position ber alle Bestellungen 
SELECT	[Bestell-Nr], [Artikel-Nr], Anzahl, 
		SUM(Anzahl) OVER(ORDER BY [Artikel-Nr]
						 ROWS BETWEEN CURRENT ROW AND 1 FOLLOWING) As SumAnzahl
FROM	dbo.Bestelldetails
WHERE	[Bestell-Nr] BETWEEN 11075 AND 11076


------------------------------
-- Fensterfunktionen mit RANGE
------------------------------

-- Summe der Anzahl aller Positionen einer Bestellung - im Vergleich mit ROWS und RANGE
SELECT	[Bestell-Nr], [Artikel-Nr], Anzahl, 
		SUM(Anzahl) OVER(PARTITION BY [Bestell-Nr] 
						 ORDER BY [Bestell-Nr] 
						 ROWS BETWEEN CURRENT ROW AND UNBOUNDED FOLLOWING) As SumAnzahlRows,
		SUM(Anzahl) OVER(PARTITION BY [Bestell-Nr] 
						 ORDER BY [Bestell-Nr] 
						 RANGE BETWEEN CURRENT ROW AND UNBOUNDED FOLLOWING) As SumAnzahlRange
FROM	dbo.Bestelldetails 
WHERE	[Bestell-Nr] BETWEEN 11075 AND 11076


-- Fehler bei RANGE in Kombination mit Einzelschritten
SELECT	[Bestell-Nr], [Artikel-Nr], Anzahl, 
		SUM(Anzahl) OVER(PARTITION BY [Bestell-Nr] 
						 ORDER BY [Bestell-Nr] 
						 RANGE BETWEEN CURRENT ROW AND 1 FOLLOWING) As SumAnzahl
FROM	dbo.Bestelldetails 
WHERE	[Bestell-Nr] BETWEEN 11075 AND 11076
