-- T-SQL - Demo 2
-- Analytische Funktionen - LAG, LEAD, FIRST_VALUE und LAST_VALUE


USE SNEK
GO

--------------------------------------
-- Inhalt der Beispieltabelle anzeigen
--------------------------------------
SELECT Jahr, Monat, Bestellsumme, JahrMonat FROM dbo.Bestellsummen


------
-- LAG
------

-- Bestellsummen inklusive der Bestellsumme vom Vormonat 
-- 1 fr den Turnus, 0 als Standardwert
SELECT	Jahr, Monat, Bestellsumme, 
		LAG(Bestellsumme, 1, 0) OVER (ORDER BY JahrMonat) As VglVorherigerMonat
FROM	dbo.Bestellsummen


-- Bestellsummen inklusive der Bestellsumme vom Vorjahr 
-- 12 fr den Turnus, 0 als Standardwert
SELECT	Jahr, Monat, Bestellsumme, 
		LAG(Bestellsumme, 12, 0) OVER (ORDER BY JahrMonat) As VglLetztesJahr
FROM	dbo.Bestellsummen


-- Bestellsummen inklusive der Bestellsumme vom Vorjahr 
-- 12 fr den Turnus, NULL als Standardwert
SELECT	Jahr, Monat, Bestellsumme, 
		LAG(Bestellsumme, 12, NULL) OVER (ORDER BY JahrMonat) As VglLetztesJahr
FROM	dbo.Bestellsummen


-- Bestellsummen pro Jahr mit Vergleich zum Vorjahr 
SELECT	Jahr, Sum(Bestellsumme) As Bestellsumme,
		LAG(Sum(Bestellsumme), 1, 0) OVER (ORDER BY Jahr) As VglLetztesJahr
FROM	dbo.Bestellsummen
GROUP BY Jahr


-------
-- LEAD
-------

-- Bestellsummen pro Jahr mit Vergleich zum Folgejahr 
-- 1 fr den Turnus, 0 als Standardwert
SELECT	Jahr, Sum(Bestellsumme) As Bestellsumme,
		LEAD(Sum(Bestellsumme), 1, 0) OVER (ORDER BY Jahr) As VglFolgendenJahr
FROM	dbo.Bestellsummen
GROUP BY Jahr


--------------
-- FIRST_VALUE
--------------

-- Bestellungen inklusive dem ersten Bestelldatum vom Kunden "ALFKI" 
SELECT	[Kunden-Code], [Bestell-Nr], Bestelldatum, 
		FIRST_VALUE(Bestelldatum) OVER (ORDER BY Bestelldatum) As Erste
FROM	dbo.Bestellungen 
WHERE	[Kunden-Code] = 'ALFKI'


-- Bestellungen inklusive dem ersten Bestelldatums pro Jahr vom Kunden "ALFKI" 
SELECT	[Kunden-Code], [Bestell-Nr], Bestelldatum, 
		FIRST_VALUE(Bestelldatum) OVER (PARTITION BY Year(Bestelldatum) ORDER BY Bestelldatum) As Erste
FROM	dbo.Bestellungen 
WHERE	[Kunden-Code] = 'ALFKI'


-------------
-- LAST_VALUE
-------------

-- Bestelldatums inklusive dem letzten Bestelldatum vom Kunden "ALFKI" 
SELECT	[Kunden-Code], [Bestell-Nr], Bestelldatum, 
		LAST_VALUE(Bestelldatum) OVER (ORDER BY Bestelldatum RANGE BETWEEN UNBOUNDED PRECEDING AND UNBOUNDED FOLLOWING) As Letzte
FROM	dbo.Bestellungen 
WHERE	[Kunden-Code] = 'ALFKI'