-- T-SQL - Demo 3
-- Analytische Funktionen - CUME_DISC, PERCENT_DISC, PERCENTILE_CONT und PERCENTILE_DISC


USE SNEK
GO


------------
-- CUME_DISC
------------

-- Relative Position des Werts "Anzahl" in den Bestellpositionen vom Kunden "ALFKI" im Jahr 2012 
SELECT	B.[Bestell-Nr], Anzahl,
		CUME_DIST() OVER(ORDER BY [Anzahl]) AS CDist
FROM	dbo.Bestelldetails BD INNER JOIN dbo.Bestellungen B 
		ON BD.[Bestell-Nr] = B.[Bestell-Nr] 
WHERE	[Kunden-Code] = 'ALFKI' AND Year(Bestelldatum) = 2012
ORDER BY Anzahl DESC
-- Formel: (Zeile + Anzahl noch folgender Zeilen) / Anzahl Zeilen gesamt
-- Berechnung CDist mit Wert "1":	(1+5) / 6 = 1
-- Berechnung CDist mit Wert "0.5":	(1+2) / 6 = 0.5


---------------
-- PERCENT_RANK 
---------------

-- Relativer Rang des Werts "Anzahl" in den Bestellpositionen vom Kunden "ALFKI" im Jahr 2012 
SELECT	B.[Bestell-Nr], [Artikel-Nr], Anzahl,
		RANK() OVER(ORDER BY Anzahl DESC) As BekannterRank,
		PERCENT_RANK() OVER(ORDER BY Anzahl) AS PercentRank
FROM	dbo.Bestelldetails BD INNER JOIN dbo.Bestellungen B 
		ON BD.[Bestell-Nr] = B.[Bestell-Nr] 
WHERE	[Kunden-Code] = 'ALFKI' AND Year(Bestelldatum) = 2012
ORDER BY Anzahl DESC
GO
-- Formel: (Anzahl Zeilen - Rank)/(Maximaler Rank)
-- Berechnung PERCENT_RANK mit Wert "1":	(6-1)/(5) = 1
-- Berechnung PERCENT_RANK mit Wert "0.4":	(6-4)/(5) = 0.4


--------------------------------------
-- PERCENTILE_CONT und PERCENTILE_DISC
--------------------------------------

-- PERCENTILE_DISC liefert den tatschlichen mittleren Wert
-- PERCENTILE_CONT liefert einen interpolierten mittleren Wert

-- Beispiel zur Berechnung eines 50%-Quantils - eines Medians

-- Beispieltabelle anlegen
CREATE TABLE dbo.Statistik (Gruppe char(1), Zahl int)
GO

-- Fnf Datenstze einfgen
INSERT INTO dbo.Statistik VALUES  ('A',4),('A',1),('A',10),('A',2),('A',1)

-- Median bei ungerader Anzahl 
SELECT	Gruppe, Zahl, 
		PERCENTILE_CONT(0.5) WITHIN GROUP (ORDER BY Zahl) OVER (PARTITION BY Gruppe) As MedianCont,
		PERCENTILE_DISC(0.5) WITHIN GROUP (ORDER BY Zahl) OVER (PARTITION BY Gruppe) As MedianDisc
FROM	dbo.Statistik
GO

-- Sechs Datenstze einfgen
INSERT INTO dbo.Statistik VALUES  ('B',4),('B',1),('B',10),('B',2),('B',1),('B',7)

-- Median bei gerader Anzahl 
SELECT	Gruppe, Zahl, 
		PERCENTILE_CONT(0.5) WITHIN GROUP (ORDER BY Zahl) OVER (PARTITION BY Gruppe) As MedianCont,
		PERCENTILE_DISC(0.5) WITHIN GROUP (ORDER BY Zahl) OVER (PARTITION BY Gruppe) As MedianDisc
FROM	dbo.Statistik
WHERE	Gruppe = 'B'
-- Berechnung bei PERCENTILE_CONT: Summe Zahl / Anzahl Zahlen = (2+4)/2=3


-- Median der Bestellpositionen zu den Bestellungen 10916 und 10917
SELECT	[Bestell-Nr], [Artikel-Nr], Anzahl,
		PERCENTILE_CONT(0.5) WITHIN GROUP (ORDER BY [Anzahl]) OVER (PARTITION BY [Bestell-Nr]) AS MedianCont,
		PERCENTILE_DISC(0.5) WITHIN GROUP (ORDER BY [Anzahl]) OVER (PARTITION BY [Bestell-Nr]) AS MedianDisc
FROM	dbo.Bestelldetails
WHERE	[Bestell-Nr] BETWEEN 10916 AND 10917
ORDER BY [Bestell-Nr] 
-- Berechnung bei PERCENTILE_CONT in Bestellung 10917: (1+10)/2 = 5,5